@@ -4,14 +4,24 @@
 function startFarabi(editorId) {
 
 	// Cache dialog jQuery references for later use
-	var $help_results_dialog = $('#help_results_dialog');
 	var $perl_critic_dialog = $('#perl_critic_dialog');
 	var $about_dialog = $('#about_dialog');
 	var $help_dialog = $("#help_dialog");
 
 	$("#theme_selector").change(function() {
 		var $selectedTheme = $(":selected", this);
-		editor.setOption("theme", $selectedTheme.val());
+		var theme = $selectedTheme.val();
+
+		$("head").append("<link>");
+		var css = $("head").children(":last");
+		css.attr({
+			rel:  "stylesheet",
+			type: "text/css",
+			href: "assets/codemirror/theme/" + theme + ".css"
+		});
+
+		editor.setOption("theme", theme);
+
 		humane.log("Mode changed to " + $selectedTheme.html());
 	});
 
@@ -34,28 +44,28 @@ function startFarabi(editorId) {
 		$.post('/help_search', {"topic": topic}, function(results) {
 			if(results.length > 0) {
 				humane.log("Found " + results.length + " help results for '" + topic +"'!");
-				$(".topic", $help_results_dialog).val(topic);
+				$(".topic").val(topic);
 				var html = '';
 				for(var i = 0; i < results.length; i++) {
 					html += '<option value="' + i + '">' + results[i].podname + "  (" + results[i].context + ")" +'</option>';
 				}
 
-				$(".results", $help_results_dialog)
+				$(".results")
 					.html(html)
 					.change(function() {
 						var index = $(':selected', this).val();
-						$(".preview", $help_results_dialog).html(results[index].html);
+						$(".preview").html(results[index].html);
 					}).change().show();
 			} else {
 				humane.log("No help found for '" + topic + "'");
 
-				$(".topic", $help_results_dialog).val(topic);
-				$(".results", $help_results_dialog).html('').hide();
-				$(".preview", $help_results_dialog).html('<span class="text-error">No results found!</span>');
+				$(".topic").val(topic);
+				$(".results").html('').hide();
+				$(".preview").html('<span class="text-error">No results found!</span>');
 			}
 			if(bShowDialog) {
-				$help_results_dialog.modal('show');
-				$(".topic", $help_results_dialog).select().focus();
+				$('a[href="#learn-tab"]').tab('show');
+				$(".topic").select().focus();
 			}
 		});
 	}
@@ -69,15 +79,28 @@ function startFarabi(editorId) {
 		CodeMirror.autoLoadMode(cm, modeFile);	
 	}
 
+	function plural(number) {
+		if(number == 1) {
+			return 'st';
+		} else if(number == 2) {
+			return 'nd';
+		} else {
+			return 'th';
+		}
+	}
+
 	function showEditorStats(cm) {
 		var cursor = cm.getCursor();
 		var selection = cm.getSelection();
+		var line_number = cursor.line + 1;
+		var column_number = cursor.ch + 1;
 		$('#editor_stats').html(
-			'Line: <strong>' + (cursor.line + 1) + '</strong>' +
-			', Column: <strong>' + (cursor.ch + 1) + '</strong>' +
-			', Selected: <strong>' + (selection ? selection.length : 0) + '</strong>' +
-			', Length: <strong>' + cm.getValue().length + '</strong>' +
-			', Lines: <strong>' + cm.lineCount() + '</strong>&nbsp;&nbsp;'
+			'<strong>' + line_number + '</strong>' + plural(line_number) + ' line' +
+			', <strong>' + column_number + '</strong>' + plural(column_number) + ' column' +
+			', <strong>' + cm.lineCount() + '</strong>&nbsp;lines' +
+			(selection ?  ', <strong>' + selection.length +'</strong>&nbsp;selected characters' : '')  +
+			', <strong>' + cm.getValue().length + '</strong>&nbsp;characters' +
+			', <strong>' + cm.lineCount() + '</strong>&nbsp;Lines&nbsp;&nbsp;'
 		);
 	}
 
@@ -110,9 +133,14 @@ function startFarabi(editorId) {
 			"F1": function(cm) {
 				displayHelp(cm);
 			},
-		},
-		onCursorActivity: function(cm) {
+			"F5": function(cm) {
+				$(".run_in_browser_button").click();
+			}
+		}
+	});
 
+	// Hook up with cursor activity event
+	editor.on("cursorActivity", function(cm) {
 			// Highlight active line
 			cm.setLineClass(hlLine, null, null);
 			hlLine = cm.setLineClass(cm.getCursor().line, null, "activeline");
@@ -122,31 +150,50 @@ function startFarabi(editorId) {
 
 			// Show editor statistics
 			showEditorStats(cm);
-		},
-		onChange: function(cm) {
-			podChanged(cm);
-		}
 	});
-	// Editor is by default Perl
-	changeMode(editor, 'perl');
 
-	// focus!
-	editor.focus();
+	// Hook up with cursor activity event
+	var timeoutId;
+	editor.on("change", function(cm) {
+			clearInterval(timeoutId);
+			timeoutId = setTimeout(function() {
+				podChanged(cm);
+				perl_critic();
+				pod_check();
+			}, 250);
+	});
 
-	// Show editor stats
-	showEditorStats(editor);
+	// Run these when we exit this one
+	setTimeout(function() {
+		// Editor is by default Perl
+		changeMode(editor, 'perl');
 
-	// Update POD tab
-	podChanged(editor);
+		// focus!
+		editor.focus();
+
+		// Show editor stats at startup
+		showEditorStats(editor);
+
+		// Update POD tab at startup
+		podChanged(editor);
+
+		// Run Perl::Critic at startup
+		perl_critic();
+
+		// Run Pod::Checker at startup
+		pod_check();
+	}, 0);
 
 	// Highlight active line
 	var hlLine = editor.setLineClass(0, "activeline");
 
+	$(".results").hide();
+
 	$(".run_in_browser_button").click(function() {
 		// Load Perlito - Perl 5 to JS Compiler
 		humane.log("Loading Perlito 5");
 		$.ajax({
-			url: 'assets/perlito/perlito5.js',
+			url: 'assets/perlito/perlito5.min.js',
 			dataType: "script",
 			cache: true,
 			success: function() {
@@ -170,30 +217,110 @@ function startFarabi(editorId) {
 		});
 	});
 
-	$(".perl_critic_button").click(function() {
+	var perlCriticWidgets = [];
+
+	var perl_critic = function() {
+
 		$.post('/perl_critic', {"source": editor.getValue(), "severity": $(':selected', '#critic_severity_selector').val()}, function(violations) {
+			for (var i = 0; i < perlCriticWidgets.length; i++) {
+				editor.removeLineWidget(perlCriticWidgets[i].widget);
+			}
+			perlCriticWidgets.length = 0;
+
 			if(violations.length > 0) {
-				humane.log("Found Perl::Critic violations!");
-				var html = '';
 				for(var i = 0; i < violations.length; i++) {
 					var violation = violations[i];
+					var description = violation.description;
+
+					// Add Perl::Critic violation under the editor line
+					var msg = $('<div class="farabi-error">' +
+						'<span class="farabi-error-icon">!!</span>' + 
+						description + '</div>')
+						.appendTo(document)
+						.get(0);
+					perlCriticWidgets.push({
+						violation: violation,
+						widget:    editor.addLineWidget(violation.line_number - 1, msg, {coverGutter: true, noHScroll: true}),
+						node:      msg
+					});
+				}
+
+				$('.farabi-error').click(function() {
+					
+					var violation;
+					for(var i = 0; i < perlCriticWidgets.length; i++) {
+						if(perlCriticWidgets[i].node == this) {
+							violation  = perlCriticWidgets[i].violation;
+							break;
+						}
+					} 
+					
+					if(!violation) {
+						return;
+					}
+
 					var policy = violation.policy;
-					html += '<strong>Description:</strong><br/>' + violation.description;
-					html += '<br/><strong>Line:</strong><br/>' + violation.line_number;
-					html += '<br/><strong>Explanation:</strong><br/>' + violation.explanation;
+					var html = '<strong>Explanation:</strong><br/>' + violation.explanation;
 					html += '<br/><strong>Policy:</strong><br/><a target="_blank" href="https://metacpan.org/module/' + policy + '">' + policy + '</a>';
 					html += '<br/><strong>Diagnositcs:</strong><br/>' + violation.diagnostics.replace(/\n/g, '<br/>');
-					html += '<hr/>';
-				}
-				$('.modal-body', $perl_critic_dialog).html(html);
-				$perl_critic_dialog.modal("show");
-			} else {
-				humane.log('Your code passed Perl::Critic!');
+
+					$('#myModalLabel', $perl_critic_dialog).html(
+						violation.description
+					);
+					$('.modal-body', $perl_critic_dialog).html(html);
+					$perl_critic_dialog.modal("show");
+				});
 			}
 		});
+	};
+
+	$("#critic_severity_selector").change(function() {
+		perl_critic();
 	});
 
+	var perlCheckerWidgets = [];
+
+	var pod_check = function() {
+		$.post('/pod_check', {"source": editor.getValue()}, function(problems) {
+			for (var i = 0; i < perlCheckerWidgets.length; i++) {
+				editor.removeLineWidget(perlCheckerWidgets[i].widget);
+			}
+			perlCheckerWidgets.length = 0;
+
+			if(problems.length > 0) {
+				for(var i = 0; i < problems.length; i++) {
+					var problem = problems[i];
+
+					// Add Pod::Checker warnings/error under the editor line
+					var msg = $('<div class="farabi-error">' +
+						'<span class="farabi-error-icon">!!</span>' + 
+						problem.message + '</div>')
+						.appendTo(document)
+						.get(0);
+					perlCheckerWidgets.push({
+						widget:    editor.addLineWidget(problem.line - 1, msg, {coverGutter: true, noHScroll: true}),
+					});
+				}
+			}
+		});
+	};
+
 	$('.about_button').click(function() {
+		if(typeof p5pkg == 'undefined') {
+			humane.log("Loading Perlito 5");
+			$.ajax({
+				url: 'assets/perlito/perlito5.min.js',
+				dataType: "script",
+				cache: true,
+				success: function() {
+					$('#perlito-version').html(  p5pkg[ "main" ][ "v_]"]);
+					humane.log("Perlito 5 loaded");
+				}
+			});
+		} else {
+			$('#perlito-version').html(p5pkg[ "main" ][ "v_]" ]);
+		}
+		$('#jquery-version').html($().jquery);
 		$about_dialog.modal("show");
 	});
 
@@ -204,7 +331,6 @@ function startFarabi(editorId) {
 	var onCloseFocusEditor = function () {
 		editor.focus();
 	};
-	$help_results_dialog.hide().on('hidden', onCloseFocusEditor);
 	$perl_critic_dialog.hide().on('hidden', onCloseFocusEditor);
 	$about_dialog.hide().on('hidden', onCloseFocusEditor);
 	$help_dialog.hide().on('hidden', onCloseFocusEditor);
@@ -226,10 +352,6 @@ function startFarabi(editorId) {
 		_displayHelp($(this).val(), true);
 	});;
 
-	setTimeout(function() {
-		$(".farabi-quote-of-the-day").hide("fade");
-	}, 8000);
-
 	$("#line_numbers_checkbox").change(function() {
 		editor.setOption('lineNumbers', $(this).is(':checked'));
 	});
@@ -248,8 +370,8 @@ function startFarabi(editorId) {
 function runOnPerlito(source) {
 
 	var $output = $('#output');
-	var $log = $("#log");
 
+	// CORE.print prints to output tab
 	p5pkg.CORE.print = function(List__) {
 		var i;
 		for (i = 0; i < List__.length; i++) {
@@ -257,37 +379,53 @@ function runOnPerlito(source) {
 		}
 		return true;
 	};
+
+	// CORE.warn print to output tab
 	p5pkg.CORE.warn = function(List__) {
 		var i;
 		List__.push("\n");
 		for (i = 0; i < List__.length; i++) {
-			$log.val( $log.val() + p5str(List__[i]));
+			$output.val( $output.val() + p5str(List__[i]));
 		}
 		return true;
 	};
+
+	// Define version, strict and warnings
 	p5pkg["main"]["v_^O"] = "browser";
 	p5pkg["main"]["Hash_INC"]["Perlito5/strict.pm"] = "Perlito5/strict.pm";
 	p5pkg["main"]["Hash_INC"]["Perlito5/warnings.pm"] = "Perlito5/warnings.pm";
 
-	$log.val('');
+	p5make_sub("Perlito5::IO", "slurp", function(filename) {
+		console.error('IO.slurp "' + filename + '"');
+		return 1;
+	});
+
+	p5is_file = function(filename) {
+		console.error('p5is_file "' + filename + '"');
+		return 1;
+	}
+
+	// Clear up output
 	$output.val('');
+
 	try {
 		// Compile Perl 5 source code into JavaScript
-		$log.val($log.val() + "Compiling.\n");
-		var start = new Date().getTime();
+		var start = $.now();
 		var js_source = p5pkg["Perlito5"].compile_p5_to_js([source]);
-		$log.val($log.val() + "Compilation time: " + (new Date().getTime() - start) + "ms\n");
 
 		// Run JavaScript inside your browser
-		start = new Date().getTime();
+		start = $.now();;
 		eval(js_source);
-		$log.val( $log.val() + "Running time: " + (new Date().getTime() - start) + "ms\n");
-		$('a[href="#output-tab"]').tab('show');
+
 	}
 	catch(err) {
-		$log.val("Error:\n" + err + "\nCompilation aborted.\n");
-		$('a[href="#log-tab"]').tab('show');
+		// Populate error and show error in output tab
+		$output.val("Error:\n" + err + "\nCompilation aborted.\n");
 	}
+
+	// Show output tab
+	$('a[href="#output-tab"]').tab('show');
+
 }
 
 // Start Farabi when the document loads
